@@ -1,7 +1,7 @@
 /*
  * Environment.java
  * 
- * Copyright (c) 2005,2007 Operational Dynamics Consulting Pty Ltd
+ * Copyright (c) 2005,2007-2008 Operational Dynamics Consulting Pty Ltd
  * 
  * The code in this file, and the library it is a part of, are made available
  * to you by the authors under the terms of the "GNU General Public Licence,
@@ -17,19 +17,22 @@ package org.freedesktop.bindings;
  * Retrieve values from environment.
  * 
  * @author Andrew Cowie
+ * @since 4.0.2
  */
 public class Environment
 {
     /**
      * Get an environment variable from the inherited (Linux or Unix)
-     * environment. This would wrap the deprecated {@link System#getenv()}
-     * call, but that throws Error in modern JDKs [so much for ABI stability,
-     * the bastards]. So we reimplement the same thing with our own JNI call.
+     * environment. This is here because the {@link System#getenv(String)}
+     * call was deprecated for a while and later threw Error in several JDKs
+     * [so much for ABI stability, the bastards]. So we reimplement the same
+     * thing with our own JNI call.
      * 
      * @param variableName
      *            the name of the environment variable you want to look up
      * @return the value of the environment variable, or <code>null</code>
      *         if empty or not found.
+     * @since 4.0.2
      */
     public static String getEnv(final String variableName) {
         if ((variableName == null) || (variableName.equals(""))) {
@@ -47,7 +50,30 @@ public class Environment
 
     private static native final String getenv(String variableName);
 
-    public static int getWidth() {
-        throw new UnsupportedOperationException("Don't know how to implement this, sorry");
+    /**
+     * Set a value in the environment. See {@link #getEnv(String) getEnv()}
+     * for further details.
+     * 
+     * <p>
+     * If a value of <code>null</code> is passed as the <code>value</code>,
+     * then that variable will be <b>deleted</b> from the environment.
+     * 
+     * @since 4.0.7
+     */
+    public static void setEnv(String variableName, String value) {
+        if ((variableName == null) || (variableName.equals(""))) {
+            throw new IllegalArgumentException("Can't set an empty or null environment variable");
+        }
+
+        if (value == null) {
+            unsetenv(variableName);
+        } else {
+            setenv(variableName, value);
+        }
     }
+
+    private static native final void setenv(String variableName, String value);
+
+    private static native final void unsetenv(String variableName);
+
 }
