@@ -67,7 +67,7 @@ public abstract class Object extends Proxy
      * a weak Java reference from the BindingsJavaClosure on the JNI side. See
      * bindings_java_signal.c in src/jni.
      */
-    private HashSet handlers;
+    private HashSet<Signal> handlers;
 
     protected Object(long pointer) {
         super(pointer);
@@ -215,7 +215,7 @@ public abstract class Object extends Proxy
      */
     void addHandler(Signal handler) {
         if (handlers == null) {
-            handlers = new HashSet();
+            handlers = new HashSet<Signal>();
         }
         handlers.add(handler);
     }
