@@ -1,10 +1,10 @@
 /*
  * FunctionGenerator.java
  *
- * Copyright (c) 2007 Operational Dynamics Consulting Pty Ltd
- * Copyright (c) 2007 Vreixo Formoso
+ * Copyright (c) 2007-2008 Operational Dynamics Consulting Pty Ltd
+ * Copyright (c) 2007-2008 Vreixo Formoso
  * 
- * The code in this file, and the library it is a part of, are made available
+ * The code in this file, and the program it is a part of, are made available
  * to you by the authors under the terms of the "GNU General Public Licence,
  * version 2" See the LICENCE file for the terms governing usage and
  * redistribution.
@@ -67,6 +67,15 @@ public class FunctionGenerator extends Generator
      */
     private Thing blacklistedType;
 
+    private final boolean addSentinal;
+
+    /**
+     * Whether the caller owns the return value. It is either 'f' (caller
+     * doesn't own return), 't' (caller own return) or 'l' (return type is a
+     * list/array... and only the list itself is own, not its contents)
+     */
+    private final char callerOwnsReturn;
+
     /**
      * @param data
      *            the information about the class to which the block we are
@@ -83,10 +92,13 @@ public class FunctionGenerator extends Generator
      * @param gParameters
      *            an array of String[2] arrays, listing the type and name of
      *            each parameter.
+     * @param callerOwnsReturn
+     *            whether the caller owns the returned value.
      */
     public FunctionGenerator(final DefsFile data, final String blockName, final String gReturnType,
-            final String cFunctionName, final String[][] gParameters) {
+            final String cFunctionName, final String[][] gParameters, char callerOwnsReturn) {
         super(data);
+        final int len;
 
         this.proxyType = (ProxiedThing) data.getType();
 
@@ -96,11 +108,30 @@ public class FunctionGenerator extends Generator
 
         this.nativeMethodName = cFunctionName;
 
-        parameterTypes = new Thing[gParameters.length];
-        parameterNames = new String[gParameters.length];
-        parameterNullOk = new boolean[gParameters.length];
+        this.callerOwnsReturn = callerOwnsReturn;
 
-        for (int i = 0; i < gParameters.length; i++) {
+        /*
+         * If ... is passed through as the last parameter, it means that we
+         * have a varargs and will need to a) add a NULL sentinel to the arg
+         * list, and b) chop this pseudo-parameter off the end of the array.
+         * This is a bit ugly, but the alternative was adding a boolean to
+         * *every* Generator constructor. This is better: varargs *is* a
+         * parameter type, and one we may someday choose to deal with.
+         */
+
+        if ((gParameters.length > 0) && ("...".equals(gParameters[gParameters.length - 1][0]))) {
+            this.addSentinal = true;
+            len = gParameters.length - 1;
+        } else {
+            this.addSentinal = false;
+            len = gParameters.length;
+        }
+
+        parameterTypes = new Thing[len];
+        parameterNames = new String[len];
+        parameterNullOk = new boolean[len];
+
+        for (int i = 0; i < len; i++) {
             parameterTypes[i] = Thing.lookup(gParameters[i][0]);
             parameterNames[i] = toCamel(gParameters[i][1]);
             parameterNullOk[i] = "yes".equals(gParameters[i][2]);
@@ -347,6 +378,9 @@ public class FunctionGenerator extends Generator
             out.print("\t");
             out.print(returnType.cType);
             out.print(" result;\n");
+            out.print("\t");
+            out.print(returnType.jniType);
+            out.print(" _result;\n");
         }
 
         for (int i = 0; i < parameterTypes.length; i++) {
@@ -473,6 +507,11 @@ public class FunctionGenerator extends Generator
             }
 
         }
+
+        if (addSentinal) {
+            out.print(", NULL");
+        }
+
         out.print(");\n");
     }
 
@@ -550,19 +589,44 @@ public class FunctionGenerator extends Generator
 
         if (!returnType.jniType.equals("void")) {
             out.print("\n");
-            out.print("\t// and finally\n");
-
-            out.print("\treturn ");
-
+            out.print("\t// translate return value to JNI type\n");
+            out.print("\t_result = ");
             out.print("(");
             out.print(returnType.jniType);
             out.print(") ");
             out.print(returnType.jniReturnEncode("result"));
             out.print(";\n");
+
+            jniFunctionReturnCleanUp(out);
+
+            out.print("\n");
+            out.print("\t// and finally\n");
+            out.print("\treturn _result;\n");
         }
         out.print("}\n");
     }
 
+    protected void jniFunctionReturnCleanUp(PrintWriter out) {
+
+        String cleanup;
+        cleanup = returnType.jniReturnCleanup("result", callerOwnsReturn);
+
+        /*
+         * TODO this assumes all type that need cleanup can be compared with
+         * NULL.
+         */
+        if (cleanup != null) {
+            out.print("\n");
+            out.print("\t// cleanup return value\n");
+            out.print("\tif (result != NULL) {\n");
+            out.print("\t");
+            out.print("\t");
+            out.print(cleanup);
+            out.print(";\n");
+            out.print("\t}\n");
+        }
+    }
+
     /**
      * Quickly scan the type information to see if there is a blacklistedType
      * type present. If so, we use that to output a throws declaration instead
