@@ -3,7 +3,7 @@
  *
  * Copyright (c) 2007 Operational Dynamics Consulting Pty Ltd
  * 
- * The code in this file, and the library it is a part of, are made available
+ * The code in this file, and the suite it is a part of, are made available
  * to you by the authors under the terms of the "GNU General Public Licence,
  * version 2" See the LICENCE file for the terms governing usage and
  * redistribution.
@@ -43,6 +43,7 @@ public class ValidateFileChoosing extends TestCaseGtk
         String retreived;
 
         fcd = new FileChooserDialog("", null, FileChooserAction.OPEN);
+        cycleMainLoop();
         fcd.setCurrentFolder("/");
         cycleMainLoop();
         assertEquals("/", fcd.getCurrentFolder());
