@@ -3,7 +3,7 @@
  *
  * Copyright (c) 2007 Operational Dynamics Consulting Pty Ltd
  * 
- * The code in this file, and the library it is a part of, are made available
+ * The code in this file, and the suite it is a part of, are made available
  * to you by the authors under the terms of the "GNU General Public Licence,
  * version 2" See the LICENCE file for the terms governing usage and
  * redistribution.
@@ -32,6 +32,9 @@ public class ValidatePacking extends TestCaseGtk
         x = new VBox(false, 3);
 
         b = new Button("Change Me?");
+
+        assertNull(b.getParent());
+
         x.packStart(b);
 
         w.add(x);
@@ -45,4 +48,25 @@ public class ValidatePacking extends TestCaseGtk
 
         assertNull("A top level Widget should have had null as its parent!", w.getParent());
     }
+
+    public final void testWidgetContainerHierarchies() {
+        final Button b;
+        final VBox x;
+        final Window w;
+
+        b = new Button("Hello");
+        x = new VBox(false, 0);
+
+        assertSame(b, b.getToplevel());
+        assertSame(x, x.getToplevel());
+
+        x.add(b);
+        assertSame(x, b.getToplevel());
+
+        w = new Window();
+        w.add(x);
+        assertSame(w, b.getToplevel());
+        assertSame(w, x.getToplevel());
+        assertSame(w, w.getToplevel());
+    }
 }
