@@ -1,9 +1,9 @@
 /*
  * ValidateProperties.java
  *
- * Copyright (c) 2007 Operational Dynamics Consulting Pty Ltd
+ * Copyright (c) 2007-2008 Operational Dynamics Consulting Pty Ltd
  * 
- * The code in this file, and the library it is a part of, are made available
+ * The code in this file, and the suite it is a part of, are made available
  * to you by the authors under the terms of the "GNU General Public Licence,
  * version 2" See the LICENCE file for the terms governing usage and
  * redistribution.
@@ -28,6 +28,11 @@ public class ValidateProperties extends TestCaseGtk
         assertEquals(-42, v.getInteger());
     }
 
+    public final void testLongValues() {
+        final Value v = new Value(600000000000L);
+        assertEquals(600000000000L, v.getLong());
+    }
+
     public final void testBooleanValues() {
         final Value v1, v2;
 
@@ -141,4 +146,79 @@ public class ValidateProperties extends TestCaseGtk
         assertEquals(0.0f, Alignment.TOP, 0.0001f);
         assertEquals(1.0f, Alignment.BOTTOM, 0.0001f);
     }
+
+    public final void testButtonImageMethods() {
+        final Button b;
+        final Image i;
+
+        b = new Button("Hello There!");
+        assertNull(b.getImage());
+
+        // doesn't matter that it'll be the "broken image" icon as a result
+        i = new Image("/dev/null");
+
+        b.setImage(i);
+        assertNotNull(b.getImage());
+        assertEquals(i, b.getImage());
+    }
+
+    public final void testAllocationLive() {
+        final Window w;
+        final Button b;
+        Allocation size;
+
+        b = new Button("Blah");
+        size = b.getAllocation();
+
+        /*
+         * These were derived empirically. A bit unexpected, but hey.
+         */
+        assertEquals(1, size.getWidth());
+        assertEquals(1, size.getHeight());
+        assertEquals(-1, size.getX());
+        assertEquals(-1, size.getY());
+
+        w = new Window();
+        w.add(b);
+        w.showAll();
+
+        /*
+         * Now test that the our live reference to the GtkAllocation actually
+         * changed. Note that we did NOT get a new Allocation object.
+         */
+        assertTrue(size.getWidth() > 1);
+        assertTrue(size.getHeight() > 1);
+        assertTrue(size.getX() >= 0);
+        assertTrue(size.getY() >= 0);
+
+        w.hide();
+
+        /*
+         * and is it indeed still the same pointer, ie the same address, ie
+         * the same Proxy?
+         */
+        assertSame(size, b.getAllocation());
+    }
+
+    public final void testRequisitionSizeRequest() {
+        final Button b;
+        Requisition req;
+        int width, height;
+
+        b = new Button("Blah");
+
+        /*
+         * This isn't quite as isolated a test as it might be; because we've
+         * got this calling gtk_widget_size_request() on the C side if a
+         * request isn't set, we can't check that it changed due to invoking
+         * this.
+         */
+
+        req = b.getRequisition();
+
+        width = req.getWidth();
+        height = req.getHeight();
+        assertTrue(width > 1);
+        assertTrue(height > 1);
+    }
 }
