@@ -3,16 +3,18 @@
  *
  * Copyright (c) 2006-2007 Operational Dynamics Consulting Pty Ltd
  *
- * The code in this file, and the library it is a part of, are made available
+ * The code in this file, and the program it is a part of, are made available
  * to you by the authors under the terms of the "GNU General Public Licence,
  * version 2" See the LICENCE file for the terms governing usage and
  * redistribution.
  */
 
+import java.io.FileNotFoundException;
+
 import org.gnome.gdk.Event;
+import org.gnome.gdk.Pixbuf;
 import org.gnome.gdk.Screen;
 import org.gnome.gtk.Button;
-import org.gnome.gtk.FileChooser;
 import org.gnome.gtk.FileChooserAction;
 import org.gnome.gtk.FileChooserButton;
 import org.gnome.gtk.Gtk;
@@ -46,6 +48,7 @@ public final class Experiment
         final Button b;
         final FileChooserButton fcb;
         final Screen s;
+        final Pixbuf logo;
 
         w = new Window();
 
@@ -65,7 +68,7 @@ public final class Experiment
 
         w.add(x);
 
-        w.setTitle("Exp");
+        w.setTitle("Experiments");
         w.showAll();
 
         s = w.getScreen();
@@ -85,14 +88,21 @@ public final class Experiment
             }
         });
 
-        fcb.connect(new FileChooser.SELECTION_CHANGED() {
-            public void onSelectionChanged(FileChooser source) {
+        fcb.connect(new FileChooserButton.FILE_SET() {
+            public void onFileSet(FileChooserButton source) {
                 System.out.println("File selected:   " + source.getURI());
                 System.gc();
             }
         });
 
         b.setTooltipText("Pressing this Button will result in some output");
+
+        try {
+            logo = new Pixbuf("src/bindings/java-gnome_Icon.png");
+            w.setIcon(logo);
+        } catch (FileNotFoundException fnfe) {
+            System.err.println("Where's the logo?");
+        }
     }
 
     public static void main(String[] args) {
